/* Sender Policy Framework Validator
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  FILE: util.c
*  DESC: Source file for utility functions.
*
*  License:
*
*  The CodeShare Software License, Version 1.0
*
*  Copyright (c) 2004 James Couzens & Travis Anderson.  All rights
*  reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
*  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED.  IN NO EVENT SHALL THE CODESHARE ORGANIZATION OR
*  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
*  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
*  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
*  SUCH DAMAGE.
*
*/

#include <stdio.h>     /* printf */
#include <string.h>    /* snprintf */

#include "util.h"      /* spfval utility functions */
#include "spfval.h"    /* spfval utility functions */

static void _MAIN_usage(const char *);


/* main
*
* Author:   Travis Anderson <tanderson@codeshare.ca>
* Author:   James Couzens <jcouzens@codeshare.ca>
*
* Date:     August 19, 2004 
*
* Desc:
*           Main function.  Handles command line arguments and performs SPFv1
* validation of the input text.
*
* Date:     November 18, 2004 - James Couzens <jcouzens@codeshare.ca>
*
* Desc:
*           Switched to use _MAIN_usage and removed strcat loop in favour of 
* having the user use the '-s' argument and placing quotes around their string so
* as not to have to bother with figuring out where the user input begins and ends. 
*
*/
int main(int argc, char **argv)
{
  int i = 1;                       /* utility */

  char *rec   = NULL;              /* record buffer */
  char *tmp   = NULL;              /* cmd line arguments tmp ptr */
  char *margv = NULL;              /* cmd line arguments working ptr */

  spfval_ctx_t *ctx = NULL;        /* spfval context */

  spfval_alert_t *alert = NULL;    /* spfval error handling */


  if (argc <= 1)
  {
    _MAIN_usage(*argv);

    return(0);
  }

  for (i = 1; i < argc; i++)
  {
    tmp = argv[i];

    if (*tmp == '-')
    {
      margv = (tmp + 3);

      switch (*(tmp + 1))
      {
        case 's':
        {
          rec = xstrndup(margv, SPFVAL_MAX_STR);
	  xvprintf("called with string: (%s)\n", rec);

          break;
        }
        case 'v':
        {
          fprintf(stderr, "Sender Policy Framework Validator (spfval) v%s\n",
            SPFVAL_VERSION);

          fprintf(stderr,
            "(c) 2004 by Travis Anderson <tanderson@codeshare.ca>\n\n");

          return(0);
        }
        default :
        {
          _MAIN_usage(*argv);
	  
	  return(-1);
        }
      }
    }
  } /* for */

  if (rec == NULL)
  {
    _MAIN_usage(*argv);

    exit(-1);
  }
    
  /* build spfval context */
  ctx = SPFVAL_new(rec);

  /* perform validation */
  if (SPFVAL_validate(ctx) == SPFVAL_TRUE)
  {
    printf("Record: (%s) parsed successfully and is considered valid\n",
      rec);
  }
  else
  {
    printf("Record: (%s) did not parse successfully\n", rec);
  }

  /* raise any errors */
  while ((alert = SPFVAL_get_alert(ctx)) != NULL)
  {
    printf("Alert type: [%s] at position: [%i] with reason: [%s]\n",
      alert->type, (alert->offset + 1), alert->str);

    printf("Hint, %s found starting here: [%s]\n", alert->type, alert->hint);
  }

  /* free internal memory associated with parse */
  SPFVAL_free(ctx);

  /* free record passed from user input */
  xfree(rec);

  return(0);
}


/* _MAIN_usage
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   08/18/04
*
*  Desc:
*         Prints out usage information.
*
*/
static void _MAIN_usage(const char *s)
{
  fprintf(stderr, "Sender Policy Framework Validator (spfval) v%s\n", SPFVAL_VERSION);
  fprintf(stderr, "(c) 2004 by Travis Anderson <tanderson@codeshare.ca>\n\n");
  fprintf(stderr, "Usage:\n\n");
  fprintf(stderr, "%s [-v] <-s SPF string>\n\n", s);
  fprintf(stderr,
    "Example: %s -s \"v=spf1 a:libspf.org include:openspf.org ptr mx -all\"\n", s);
  fprintf(stderr, "\n");

  return;
}

/* end of main.c */
