/* Sender Policy Framework Validator
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  File:   spfval.c
*  Desc:   An SPF validation function set
*
*  License:
*
*  The CodeShare Software License, Version 1.0
*
*  Copyright (c) 2004 James Couzens & Travis Anderson  All rights
*  reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
*  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED.  IN NO EVENT SHALL CODESHARE NOR ANY OF ITS ASSOCIATED
*  PROGRAMMERS OR OTHER INDIVIDUALS AFFILIATED WITH THE OWNERSHIP OR
*  OPERATION OF CODESHARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
*  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
*  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
*  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <stdio.h>         /* printf */
#include <stdlib.h>        /* malloc / free */
#include <string.h>        /* printf */
#include <sys/socket.h>    /* inet_ntoa */
#include <netinet/in.h>    /* inet_ntoa / ntohl */
#include <arpa/inet.h>     /* inet_ntoa */

#include "spfval.h"        /* SPFVAL header file */



/*
* local global defines
*
*/

/* maximum length of an IPv4 IP address */
#define MAX_IP_ADDR 17


/*
* local global variables
*
*/


/* spfval_alert_type_map
*
*  Desc:
*         each of these strings matches up (in order) with an ID from 
*  the 'SPFVAL_ALERT_TYPE' enumeration and can be uysed to easily obtain
*  a human readable string when one has obtained an appropriate ALERT ID.
*
*/
static char *spfval_alert_type_map[] =
{
  "error",
  "warning",
};


/* spfval_err_str_map
*
*  Desc:
*         each of these strings matches up (in order) with an ID from
*  the 'SPFVAL_ERROR_ID' enumeration  and can be used to easily obtain
*  a human readable string when one has an ERROR ID.
*
*/
static char *spfval_err_str_map[] =
{
  "String does not appear to be an SPF record",
  "Not enough mechanisms in string to be an SPF record",
  "Prefixes cannot be applied to modifiers",
  "Unrecognized mechanism",
  "This directive cannot contain a macro",
  "Macro is not terminated",
  "This macro is not valid in this mechanism",
  "Invalid macro command",
  "Invalid macro transformer",
  "Invalid macro delimiter",
  "Found trailing characters on macro",
  "Modifier must have an argument",
  "A /cidr is only applicable in a: mx: ip4: and ip6:",
  "exists: MUST have an argument",
  "Arguments are not applicable to all:",
  "No mechanisms or modifiers; record has no effect",
  "Macro is empty; has no effect",
  "Hint, error occurred starting here: %s"
};


/* spfval_warn_str_map
*
*  Desc:
*         each of these strings matches up (in order) with an ID from 
*  the 'SPFVAL_WARN_ID' enumeration and can be used to easily obtain a
*  human readable string when one has an obtained a WARNING ID.
*
*/
static char *spfval_warn_str_map[] =
{
  "Modifier does not conform to spf1 (extension? deprecated?)",
  "Unescaped percent; should be '%%' not '%'",
  "Hint, warning occurred starting here: %s"
};


/* spf_mech_macro_map
*
*  Desc:
*         This mapping of integers affords the functionality of being
*  able to easily tell if a particular macro is permitted within a given
*  mechanism.
*
*/
static char spf_mech_macro_map[] =
{
  SPFVAL_FALSE,   /* MECH_ALL */
  SPFVAL_TRUE,    /* MECH_INCLUDE */
  SPFVAL_FALSE,   /* MECH_A */
  SPFVAL_FALSE,   /* MECH_MX */
  SPFVAL_FALSE,   /* MECH_PTR */
  SPFVAL_FALSE,   /* MECH_IP4 */
  SPFVAL_FALSE,   /* MECH_IP6 */
  SPFVAL_TRUE,    /* MECH_EXISTS */
};



/* _SPFVAL_is_prefix
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Determines if the char 'prefix' is an SPF prefix. 
*
*  Returns SPFVAL_TRUE if it is, and SPFVAL_FALSE if it isn't.
*      
*  Date:   August 27, 2004
*
*  Desc:
*          Changed default case to 'break' from return(0); and moved that
*  down to be the return statement for the function.
*
*  Changed the functions return type from 'int' to 'SPFVAL_BOOL' and 
*  changed the various returns accordingly.
*
*  Changed passed type from 'char' to 'const char'
*
*/
static SPFVAL_BOOL _SPFVAL_is_prefix(const char prefix)
{
  switch (prefix)
  {
    case '+':
    {
      return(SPFVAL_TRUE);    /* pass */
    }
    case '-':
    {
      return(SPFVAL_TRUE);    /* fail */
    }
    case '~':
    {
      return(SPFVAL_TRUE);    /* neutral */
    }
    case '?':
    {
      return(SPFVAL_TRUE);    /* unknown */
    }
    default :
    {
      break;
    }
  }

  return(SPFVAL_FALSE);
}


/* _SPFVAL_is_macro_letter
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Determines if the char 'letter' is a valid SPF macro expansion
*  letter as set out by the RFC.    
*
*  Date:   August 27, 2004
*
*  Desc:
*          Changed default case to 'break' from return(0); and moved that
*  down to be the return statement for the function. 
*
*  Changed the functions return type from 'int' to 'SPFVAL_BOOL' and 
*  changed the various returns accordingly.
*
*  Changed passed type from 'char' to 'const char'
*
*/
static SPFVAL_BOOL _SPFVAL_is_macro_letter(const char letter)
{
  switch (letter)
  {
    case 'l': { }
    case 'L':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 's': { }
    case 'S':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'o': { }
    case 'O':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'd': { }
    case 'D':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'i': { }
    case 'I':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'p': { }
    case 'P':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'v': { }
    case 'V':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'h': { }
    case 'H':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'r': { }
    case 'R':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'c': { }
    case 'C':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 't': { }
    case 'T':
    {
      return(SPFVAL_TRUE);    /**/
    }
    default :
    {
      break;
    }
  }

  return(SPFVAL_FALSE);
}


/* _SPFVAL_is_macro_transformer
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codehsare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*       
*
*  Date:   August 27, 2004
*
*  Desc:
*          Changed default case to 'break' from return(0); and moved that
*  down to be the return statement for the function. 
*
*  For clarity reasons changed all the returns to 1 or 0 to reference
*  the global boolean type SPFVAL_BOOL.  Perhaps its possible to change
*  the expected return of 'r' from -1 to a SPFVAL_FALSE so that this
*  this functions return type can be changed appropriately.
*
*  changed passed type from 'char' to 'const char'
*
*/
static int32_t _SPFVAL_is_macro_transformer(const char transformer)
{
  switch (transformer)
  {
    case '0':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '1':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '2':
    { 
      return(SPFVAL_TRUE);    /**/
    }
    case '3':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '4':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '5':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '6':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '7':
    { 
      return(SPFVAL_TRUE);    /**/
    }
    case '8':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '9':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case 'r':
    {
      return(-1);             /**/
    }
    default :
    { 
      break;
    }
  }

  return(SPFVAL_FALSE);
}


/* _SPFVAL_is_macro_delimiter
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*         
*           
*
*  Date:   August 27, 2004
*
*  Desc:
*          Changed default case to 'break' from return(0); and moved that
*  down to be the return statement for the function.
*
*  Changed the functions return type from 'int' to 'SPFVAL_BOOL' and 
*  changed the various returns accordingly.
*
*  changed passed type from 'char' to 'const char'
*
*/
static SPFVAL_BOOL _SPFVAL_is_macro_delimiter(const char delimiter)
{
  switch (delimiter)
  {
    case '.':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '-':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '+':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case ',':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '/':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '_':
    {
      return(SPFVAL_TRUE);    /**/
    }
    case '=':
    {
      return(SPFVAL_TRUE);    /**/
    }
    default :
    {
      break;
    }
  }

  return(SPFVAL_FALSE);
}


/* _SPFVAL_get_mechanism_id
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004   
*
*  Desc:
*          Obtains the ID associated wit the SPF mechanism ID contained within 
*  the passed char 'mech' and returns the appropriate enumerated type from
*  the enumration 'SPFVAL_MECHANISM'.
*
*  Returns -1 upon an inability to discover the ID.
*       
*
*/
static int32_t _SPFVAL_get_mechanism_id(const char *mech)
{
  if (!strcasecmp(mech, "all"))
  {
    return(MECH_ALL);
  }
  else if (!strcasecmp(mech, "include")) 
  {
    return(MECH_INCLUDE);
  }
  else if (!strcasecmp(mech, "a"))
  {
    return(MECH_A);
  }
  else if (!strcasecmp(mech, "mx"))
  {
    return(MECH_MX);
  }
  else if (!strcasecmp(mech, "ptr"))
  {
    return(MECH_PTR);
  }
  else if (!strcasecmp(mech, "ip4"))
  {
    return(MECH_IP4);
  }
  else if (!strcasecmp(mech, "ip6"))
  {
    return(MECH_IP6);
  }
  else if (!strcasecmp(mech, "exists"))
  {
    return(MECH_EXISTS);
  }
  else
  {
    return(-1);
  }
}


/* _SPFVAL_get_modifier_id
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004  
*
*  Desc:
*          Obtains the ID associated with the SPF modifier ID contained within
*  the passed char 'mod' and returns the appropriate enumerated type from the
*  enumeration 'SPFVAL_MODIFIER'.
*
*  Return:
*           -1 upon an inability to discover the ID.
*
*/
static int32_t _SPFVAL_get_modifier_id(const char *mod)
{
  if (!strcasecmp(mod, "v"))
  {
    return(MOD_VERSION);
  }
  else if (!strcasecmp(mod, "redirect"))
  {
    return(MOD_REDIRECT);
  }
  else if (!strcasecmp(mod, "exp"))
  {
    return(MOD_EXPLAIN);
  }
  else
  {
    return(-1);
  }
}


/* _SPFVAL_get_hint
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   November 18, 2004
*
*  Desc:
*          Using the passed type and result code a string is allocated
*  and then populated with what could best be described as a human readable
*  explanation for the error or warning defined by the passed type and code.
*  This function was originally written to simply return the remainder of 
*  the original record in newly allocated memory starting at the offset,
*  but could easily be altered to do more (and it is hoped that this will
*  be so).
*
*  Return:
*          NULL upon failure (for whatever reason) and a string of allocated
*  memory upon success
*
*/
static char *_SPFVAL_get_hint(spfval_ctx_t *ctx)
{
  char *ret_ptr = NULL;    /* return pointer */
  char *tmp     = NULL;    /* working pointer */

  tmp = ctx->record;
  tmp += ctx->offset;

  ret_ptr = xstrndup(tmp, SPFVAL_MAX_STR);

  xvprintf("returning: (%s)\n", ret_ptr);

  return(ret_ptr);
}


/* _SPFVAL_alert
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          This function builds a linked list of either of two types of
*  'alerts', both of which are considered to be bad, however one is more
*  fatal than the other.  The 'alerts' are: 'warning' and 'error'.
*  This function performs the actual work of two wrapper functions
*  '_SFPVAL_error' and '_SPFVAL_warning' located within this same body of
*  code.
*
*  Return:
*           SPFVAL_FALSE upon any type of failure, and SPFVAL_TRUE upon
*  success.
*
*  Date:    November 18, 2004 by James
*
*  Desc:
*           Altered function return type to be SPFVAL_BOOL and appropriately
*  adjusted any return statements.  In addition default values were supplied
*  to locally declared variables.
*
*  I also 'upgraded' the error reporting to actually move to the position 
*  in the record using the already generated offset and then displaying the
*  originally passed SPF string at the position indicated by the error
*  offset.
*
*  Removed malloc cast.
*
*/
static SPFVAL_BOOL _SPFVAL_alert(spfval_ctx_t *ctx, int32_t type, int32_t code)
{
  int32_t code_max = 0;         /* maximum alert level (working) */
 
  spfval_alert_t *va = NULL;    /* linked list */

  
  if (ctx == NULL)
  {
    return(SPFVAL_FALSE);
  }

  if (type == ALERT_ERROR)
  {
    code_max = ERR_END;
  }
  else if (type == ALERT_WARNING)
  {
    code_max = WARN_END;
  }
  else
  {
    xvprintf("Invalid alert type %i", type);

    return(SPFVAL_FALSE);
  }

  if ((code < 0) || (code >= code_max))
  {
    xvprint("Error/Warning code out of range");

    return(SPFVAL_FALSE);
  }

  va = xmalloc(sizeof(spfval_alert_t));

  va->code   = code;
  va->offset = ctx->offset;
  va->next   = NULL;
  va->hint   = NULL;
  va->str    = NULL;

  ctx->num_alerts++;

  if (type == ALERT_ERROR)
  {
    ctx->num_errors++;
    va->str  = spfval_err_str_map[code];
    va->hint = (char *)_SPFVAL_get_hint(ctx);
    va->type = spfval_alert_type_map[ALERT_ERROR];
  }
  else if (type == ALERT_WARNING)
  {
    ctx->num_warnings++;
    va->str  = spfval_warn_str_map[code];
    va->hint = (char *)_SPFVAL_get_hint(ctx);
    va->type = spfval_alert_type_map[ALERT_WARNING];
  }
  else
  {
    return(SPFVAL_FALSE);
  }

  if (ctx->alert_head == NULL)
  {
    ctx->alert_head = va;
    ctx->alert_tail = va;

    return(SPFVAL_TRUE);
  }
  else
  {
    ctx->alert_tail->next = va;
    ctx->alert_tail       = va;

    return(SPFVAL_TRUE);
  }
}


/* _SPFVAL_error
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Reports Errors.
*       
*
*/
static int32_t _SPFVAL_error(spfval_ctx_t *ctx, int32_t err_code)
{
  return(_SPFVAL_alert(ctx, ALERT_ERROR, err_code));
}


/* _SPFVAL_warning
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*       
*
*/
static int32_t _SPFVAL_warning(spfval_ctx_t *ctx, int32_t warn_code)
{
  return(_SPFVAL_alert(ctx, ALERT_WARNING, warn_code));
}


/* _SPFVAL_macro
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*       
*
*  Date:   August 27, 2004
*
*  Desc:
*          Changed the comparison of '1' to 'SPFVAL_TRUE' where the function
*  _SPFVAL_is_macro_letter is concerned
*
*  Date:    November 18, 2004
*
*  Desc:
*          Changed the function return type to be SPFVAL_BOOL and appropriately
*  adjusted any return statements to reflect this.  In addition supplied locally
*  declared variables with default values.
*
*/
static SPFVAL_BOOL _SPFVAL_macro(spfval_ctx_t *ctx, const char *macro,
  const char *directive)
{
  int32_t i     = 0;    /* utility */
  int32_t state = 0;    /* macro state */
  
  char *p = NULL;       /* working pointer */

  enum
  {
    IN_LETTER = 0,
    IN_TRANSFORMER,
    IN_DELIMITER
  };

  
  if ((ctx == NULL) || (macro == NULL) || (directive == NULL))
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  state = 0;
  p = (char *)macro;

  if (*p == 0)
  {
    xvprint("Found an empty macro");

    _SPFVAL_error(ctx, ERR_EMPTY_MACRO);
  }

  while (*p)
  {
    if (state == IN_LETTER)
    {
      if (_SPFVAL_is_macro_letter(*p) == SPFVAL_TRUE)
      {
        if ((*p == 'c') || (*p == 't'))
        {
          if (strcasecmp(directive, "exp"))
          {
            /* these two arent valid for anything other than exp= */
            xvprintf("The '%c' macro may only be used with 'exp='", *p);

            _SPFVAL_error(ctx, ERR_DIRECTIVE_CANNOT_CONTAIN_MACRO);
          }
        }
        state++;
      }
      else
      {
        if ((*p == '%') || (*p == '_') || (*p == '-'))
        {
          ctx->offset++;

          return(SPFVAL_TRUE);
        }
        else
        {
          xvprintf("Expected a valid macro command, but got '%c'", *p);

          _SPFVAL_error(ctx, ERR_INVALID_MACRO_COMMAND);
          state++;
        }
      }
    }
    else if (state == IN_TRANSFORMER)
    {
      if ((i = _SPFVAL_is_macro_transformer(*p)) == 0)
      {
        state++;

        continue;
      }
      else if (i == -1)
      {
        state++;
      }
    }
    else if (state == IN_DELIMITER)
    {
      if (_SPFVAL_is_macro_delimiter(*p) != SPFVAL_TRUE)
      {
        xvprintf("Expected a valid macro delimiter, but got '%c'", *p);

        _SPFVAL_error(ctx, ERR_INVALID_MACRO_DELIMITER);
      }
    }
    else
    {
      xvprint("Extra characters found after macro delimiter");

      _SPFVAL_error(ctx, ERR_TRAILING_CHARS_ON_MACRO);
    }

    ctx->offset++;
    p++;
  }

  return(SPFVAL_TRUE);
}


/* _SPFVAL_get_macro
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Obtain a macro from a passed buffer
*
*  Return: 
*          SPFVAL_TRUE upon success (obtained a macro) and SPFVAL_FALSE upon
*  failure (not valid macro could be found).
*
*  Date:   November 18, 2004
*
*  Desc:
*          Changed function return type to be SPFVAL_BOOL and appropriately
*  adjusted any return statements in addition to supplying locally declared
*  variables with default values.
*
*/
static SPFVAL_BOOL _SPFVAL_get_macro(spfval_ctx_t *ctx, char **ptr, char *buf)
{
  int32_t i = 0;      /* utility */
  
  char *p = NULL;     /* working pointer */

  
  if ((ctx == NULL) || (ptr == NULL) || (*ptr == NULL) || (buf == NULL))
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  i = 0;
  p = *ptr;

  p = strchr(p, '%');

  if (p != NULL)
  {
    ctx->offset += (p - *ptr + 1);
    p++;

    if ((*p == '%') || (*p == '_') || (*p == '-'))
    {
      xvprintf("Found macro: %%%c", *p);

      buf[i++] = *p;
      buf[i]   = 0;

      p++;
      *ptr = p;

      return(SPFVAL_TRUE);
    }

    if (*p != '{')
    {
      ctx->offset--;
      xvprint("Un-escaped percent; but treated as literal");

      _SPFVAL_warning(ctx, WARN_UNESCAPED_PERCENT);

      ctx->offset++;

      buf[i] = 0;
      *ptr   = p;

      return(SPFVAL_TRUE);
    }

    ctx->offset++;
    p++;

    while (*p != '}')
    {
      if (*p == 0)
      {
        xvprint("Got \\0 in macro before '}' was found");

        _SPFVAL_error(ctx, ERR_MACRO_NOT_TERMINATED);

        break;
      }

      buf[i++] = *p;
      p++;
    }

    buf[i] = 0;
    *ptr   = p;

    xvprintf("Found macro: %%{%s}", buf);

    return(SPFVAL_TRUE);
  }

  p = *ptr;

  while (*p)
  {
    ctx->offset++;
    p++;
  }

  *ptr = p;

  xvprint("Could not retrieve a macro");

  return(SPFVAL_FALSE);
}


/* _SPFVAL_modifier
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Operates on a modifer 
*
*  Return:
*          SPFVAL_TRUE upon success (modifier is valid) and SPFVAL_FALSE in
*  the event of failure (modifier is NOT valid).
*
*  Date:   November 18, 2004 by James
*
*  Desc:
*          Altered function return type to be SPFVAL_BOOL and appropriately
*  adjusted all function return statements as well as supplying declared local
*  variables with default values.
*
*/
static SPFVAL_BOOL _SPFVAL_modifier(spfval_ctx_t *ctx, const char *mod,
  const char *arg, int32_t add)
{
  int32_t res = 0;                     /* result integer code */
  
  char macro[MAX_RECORD_LEN] = {0};    /* macro storage */
  
  char *p = NULL;                      /* working pointer */

  
  if ((ctx == NULL) || (mod == NULL) || (arg == NULL))
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  res = _SPFVAL_get_modifier_id(mod);

  if ((res < 0) || (res >= MOD_END))
  {
    xvprintf("Modifier not recognized: %s", mod);

    _SPFVAL_warning(ctx, WARN_MODIFIER_DOESNT_CONFORM);
  }
  else
  {
    if (res == MOD_REDIRECT)
    {
      ctx->found_redirect = 1;
    }
  }

  ctx->offset += (strlen(mod) + add);
  p = (char *)arg;

  if (strlen(arg) == 0)
  {
    xvprint("Modifier must have an argument");

    _SPFVAL_error(ctx, ERR_MODIFIER_HAS_NO_ARGUMENT);

    return(SPFVAL_FALSE);
  }

  while (_SPFVAL_get_macro(ctx, &p, macro))
  {
    _SPFVAL_macro(ctx, macro, mod);
  }

  return(SPFVAL_TRUE);
}


/* _SPFVAL_mechanism
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Operates on a mechnaism to validate it
*
*  Date:   November 18, 2004 by James
*
*  Desc:
*          Changed function return type to SPFVAL_BOOL and appropriately
*  adjusted any return statements.  Also applied default values to all declared
*  variables.
*
*/
static SPFVAL_BOOL _SPFVAL_mechanism(spfval_ctx_t *ctx, const char *mech,
  const char *arg, int32_t add)
{
  int32_t res       = 0;               /* result code */
  int32_t mech_type = 0;               /* mechanism type */
  
  char macro[MAX_RECORD_LEN] = {0};    /* macro */
  
  char *p = NULL;                      /* working pointer */

  
  if (mech == NULL)
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  xvprintf("Mechanism: '%s'", mech);

  res = _SPFVAL_get_mechanism_id(mech);
  if ((res < 0) || (res >= MECH_END))
  {
    /* invalid mech */
    xvprintf("Mechanism not recognized: %s", mech);

    _SPFVAL_error(ctx, ERR_MECHANISM_UNRECOGNIZED);
  }
  else
  {
    if (res == MECH_INCLUDE)
    {
      ctx->found_include = 1;
    }
    else if (res == MECH_EXISTS)
    {
      if (*arg == 0)
      {
        xvprint("exists: MUST have an argument");

        _SPFVAL_error(ctx, ERR_EXISTS_MUST_HAVE_ARGUMENT);
      }
    }
    else if (res == MECH_ALL)
    {
      if (*arg != 0)
      {
        xvprint("arguments are not applicable to all:");

        _SPFVAL_error(ctx, ERR_ARGUMENT_NOT_APPLICABLE);
      }
    }
  }

  if (*arg == '/')
  {
    if ((res != MECH_A) && (res != MECH_MX) &&
        (res != MECH_IP4) && (res != MECH_IP6))
    {
      xvprint("a /cidr is only applicable in a: mx: ip4: and ip6:");

      _SPFVAL_error(ctx, ERR_CIDR_NOT_APPLICABLE);
    }
  }

  mech_type = res;
  p = (char *)arg;

  if (arg != NULL)
  {
    ctx->offset += strlen(mech) + add;

    while (_SPFVAL_get_macro(ctx, &p, macro))
    {
      if (!spf_mech_macro_map[mech_type])
      {
        xvprintf("Mechanism %s may not contain macros", mech);

        _SPFVAL_error(ctx, ERR_DIRECTIVE_CANNOT_CONTAIN_MACRO);
      }

      _SPFVAL_macro(ctx, macro, mech);
    }
  }
  else
  {
    ctx->offset = strlen(mech);
  }

  return(SPFVAL_TRUE);
}


/* _SPFVAL_token
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Operates on a 'token' of the original record and validates
*  it.
*
*  Return:
*          SPFVAL_TRUE upon success (valid token) or SPFVAL_FALSE if not.
*
*  Date:   August 27, 2004 by James
*
*  Desc:
*          Switched out is_mech/is_mod/must_be_mech to be of type SPFVAL_BOOL
*  instead of signed ints and made appropriate changes elsewhere to the
*  function to reflect this.
*
*  Placed brackets around various calculations for clarity reasons.
*
*  Date:   November 18, 2004
*
*  Desc:
*          Changed function return type to SFPVAL_BOOL and adjusted any
*  return statements appropriately.  Also applied default values to all 
*  declared variables.
*
*/
static int32_t _SPFVAL_token(spfval_ctx_t *ctx, const char *token)
{
  int32_t add = 0;                            /**/
  
  SPFVAL_BOOL is_mech      = SPFVAL_FALSE;    /* is a mechanism */
  SPFVAL_BOOL is_mod       = SPFVAL_FALSE;    /* is a modifier */
  SPFVAL_BOOL must_be_mech = SPFVAL_FALSE;    /* must be a mechanism */
  
  char directive[MAX_RECORD_LEN] = {0};       /* directives */
  char arg[MAX_RECORD_LEN] = {0};             /* arguments */
  
  char *tokptr = NULL;                        /* token pointer */
  char *p      = NULL;                        /* working pointer */


  if ((ctx == NULL) || (token == NULL))
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  xvprintf("Token: '%s'", token);

  tokptr = (char *)token;
  p      = (char *)token;

  if (_SPFVAL_is_prefix(*p) == SPFVAL_TRUE)
  {
    xvprintf("Found prefix '%c'; must be mechanism", *p);

    must_be_mech = SPFVAL_TRUE;

    tokptr++;
    ctx->offset++;
    p++;
  }

  /* ':', '/', '=' are valid */
  while ((*p != ':') && (*p != '/') && (*p != '=') && (*p != '\0'))
  {
    p++;
  }

  if ((*p == ':') || (*p == '/') || (*p == '\0'))
  {
    is_mech = SPFVAL_TRUE;

    if ((p - tokptr + 1) > MAX_RECORD_LEN)
    {
      xvprintf("Directive too long: %i > %i",
        (p - tokptr + 1), MAX_RECORD_LEN);

      return(SPFVAL_FALSE);
    }

    snprintf(directive, (p - tokptr + 1), "%s", tokptr);

    if (*p == ':')
    {
      add++;
      p++;
      snprintf(arg, MAX_RECORD_LEN, "%s", p);
    }
    else if (*p == '/')
    {
      snprintf(arg, MAX_RECORD_LEN, "%s", p);
    }

    xvprintf("Directive: '%s', argument: '%s'", directive, arg);
    _SPFVAL_mechanism(ctx, directive, arg, add);

    ctx->num_mechanisms++;
  }
  else if (*p == '=')
  {
    if (must_be_mech == SPFVAL_TRUE)
    {
      xvprint("Modifiers may not have prefixes");

      _SPFVAL_error(ctx, ERR_MODIFIER_WITH_PREFIX);
    }

    is_mod = SPFVAL_TRUE;

    if ((p - tokptr + 1) > MAX_RECORD_LEN)
    {
      xvprintf("Directive too long: %i > %i",
        (p - tokptr + 1), MAX_RECORD_LEN);

      return(SPFVAL_FALSE);
    }

    snprintf(directive, (p - tokptr + 1), "%s", tokptr);

    add++;
    p++;

    snprintf(arg, MAX_RECORD_LEN, "%s", p);

    xvprintf("Directive: '%s', argument: '%s'", directive, arg);

    _SPFVAL_modifier(ctx, directive, arg, add);

    ctx->num_modifiers++;
  }

  return(SPFVAL_TRUE);
}


/* _SPFVAL_get_token
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*       
*
*  Date:   August 27, 2004 by James
*
*  Desc:  
*          Placed brackets around various calculations for clarity reasons
*
*  Date:   November 18, 2004 by James
*
*  Desc:
*          Changed return type to SPFVAL_BOOL and altered all appropriate
*  return statements.
*
*/
static SPFVAL_BOOL _SPFVAL_get_token(spfval_ctx_t *ctx, char **ptr, char *buf)
{
  char *p = NULL;    /* working pointer */

  
  if ((ctx == NULL) || (ptr == NULL) || (*ptr == NULL) || (buf == NULL))
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  if (**ptr == 0)
  {
    xvprint("Empty token");

    return(SPFVAL_FALSE);
  }

  p = strchr(*ptr, ' ');
  if (p == NULL)
  {
    snprintf(buf, MAX_RECORD_LEN, "%s", *ptr);
  }
  else
  {
    if ((p - *ptr + 1) > MAX_RECORD_LEN)
    {
      xvprintf("Token too long: %i > %i", (p - *ptr + 1), MAX_RECORD_LEN);

      return(SPFVAL_FALSE);
    }

    snprintf(buf, (p - *ptr + 1), "%s", *ptr);
  }

  if (p != NULL)
  {
    /* find the start of the next token */
    while (*p == ' ')
    {
      p++;
    }

    *ptr = p;
  }
  else
  {
    p = *ptr;

    while (*p)
    {
      p++;
    }

    *ptr = p;
  }

  xvprintf("Found token: %s", buf);

  return(SPFVAL_TRUE);
}


/* _SPFVAL_is_spf_record
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*
*       
*  Date:   August 27, 2004 by James
*
*  Desc:  
*          Placed brackets around various calculations for clarity reasons
*
*  Date:   November 18, 2004 by James
*
*  Desc:
*          Altered return type to SPFVAL_BOOL and altered all appropriate
*  return statements.
*
*/
static SPFVAL_BOOL _SPFVAL_is_spf_record(spfval_ctx_t *ctx)
{
  int32_t i = 0;                     /* utility */
  
  char ver[MAX_RECORD_LEN] = {0};    /* version string */
  
  char *p = NULL;                    /* working pointer */
  char *q = NULL;                    /* secondary working pointer */


  if (ctx == NULL)
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  p = ctx->record;

  while ((p = strchr(p, 'v')) != NULL)
  {
    p++;

    if (*p == '=')
    {
      p++;
      q = strchr(p, ' ');

      if (q != NULL)
      {
        i = (q - p + 1);

        if (i > MAX_RECORD_LEN)
        {
          i = MAX_RECORD_LEN;

          xvprintf("Version string exceeded max version length: %i > %i",
            i, MAX_RECORD_LEN);

          continue;
        }

        snprintf(ver, i, "%s", p);
      }
      else
      {
        snprintf(ver, MAX_RECORD_LEN, "%s", p);
      }

      if (!strncasecmp(ver, VERSION_STRING, strlen(VERSION_STRING)))
      {
        xvprintf("Record recognized as %s", VERSION_STRING);

        return(SPFVAL_TRUE);
      }
      else
      {
        xvprintf("Record unrecognized; identified as %s", ver);
      }

      /* Comment: Travis Anderson <tanderson@codeshare.ca>
      *  Date:    August 16, 2004
      *
      * not a version string we support
      *
      * multiple version strings are invalid, but since this func is only
      * supposed to tell if something is an SPF record, we'll continue anyway
      *
      */
    }
  }

  xvprint("No version string found in record");

  return(SPFVAL_FALSE);
}



/* _SPFVAL_cidr_cmp
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   12/26/03
*
*  Desc:
*          Compares an IP address (connected peer) against another IP
*  address (found in a policy lookup) using a netmask (also found from
*  a policy lookup) to see if the peer address is legal within that
*  netmask.  Returns TRUE if its valid, FALSE if not.
*
*  Notes:  Write additional comparisons to handle IP6 addeesses
*
*/
static SPFVAL_BOOL _SPFVAL_cidr_cmp(const policy_addr_t *policy_addr,
  const struct in_addr *peer_addr)
{
  uint32_t a = 0;            /* utility */
  uint32_t b = 0;            /* utility */

  char *peer_ip   = NULL;    /* ip address of connected peer */
  char *policy_ip = NULL;    /* ip address from SPF policy */
  

  if ((policy_addr->addr.s_addr <= 0) && (peer_addr->s_addr <= 0))
  {
    xvprint("Passed with NULL chars.  Aborting.\n");

    return(SPFVAL_FALSE);
  }

  xvprintf("POL: %lu PEER: %lu CIDR: %i\n",
    policy_addr->addr.s_addr, peer_addr->s_addr, policy_addr->cidr);

  a = ntohl(peer_addr->s_addr);
  b = ntohl(policy_addr->addr.s_addr);

  if (policy_addr->cidr != 32)
  {
    if ( (a & (~0U << (32 - policy_addr->cidr))) !=
         (b & (~0U << (32 - policy_addr->cidr))) )
    {
      return(SPFVAL_FALSE);
    }
  }
  else
  {
    if (peer_addr->s_addr != policy_addr->addr.s_addr)
    {
      xvprintf("%lu and %lu using 32 cidr do not match\n",
        peer_addr->s_addr, policy_addr->addr.s_addr);

      return(SPFVAL_FALSE);
    }
  }

  /* these are done here just for debugger remove later */
  peer_ip   = xstrndup(inet_ntoa(*peer_addr), MAX_IP_ADDR);
  policy_ip = xstrndup(inet_ntoa(policy_addr->addr), MAX_IP_ADDR);

  xvprintf("Peer: (%s) matches address %s with network %i\n",
    peer_ip, policy_ip, policy_addr->cidr);

  xfree(peer_ip);
  xfree(policy_ip);

  return(SPFVAL_TRUE);
}


/* SPFVAL_new
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Allocates memory for a new spfval context and then in the event
*  of no errors encountered returns the newly allocated memory to the caller.
*
*  Return: 
*          NULL in the event of an error, and a spfval_ctx_t structure in 
*  the event of success.  Calling function is expected to call SPFVAL_free
*  on this structure to properly deallocate the memory within.
*
*
*  Date:  August 27, 2004 by James
*
*  Desc:  
*         Placed brackets around various calculations for clarity reasons
*
*
*  Date:  November 18, 2004 by James
*
*  Desc:
*         Added more debugging output
*
*/
spfval_ctx_t *SPFVAL_new(const char *record)
{
  spfval_ctx_t *ctx = NULL;    /* spfval context */


  if (record == NULL)
  {
    xvprint("NULL arguments passed");

    return(NULL);
  }

  ctx = (spfval_ctx_t *)xmalloc(sizeof(spfval_ctx_t));
  memset(ctx, 0, sizeof(spfval_ctx_t));

  ctx->record_len = strlen(record);

  if (ctx->record_len > (MAX_RECORD_LEN - 1))
  {
    xvprint("Record length too long");

    return(NULL);
  }

  ctx->record = xmalloc(ctx->record_len + 1);

  snprintf(ctx->record, (ctx->record_len + 1), "%s", record);

  xvprintf("returning context with record: (%s)\n", ctx->record);

  return(ctx);
}


/* SPFVAL_free
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Frees memory allocated within the spfval_ctx_t structure
*  safely and rethrns nothing.  Success is either always assumed or
*  any possible failure is ignored with the mindset that if you can't
*  safely free memory you have greater problems with your system =D
*
*  Return:
*          N/A
*
*/
void SPFVAL_free(spfval_ctx_t *ctx)
{
  spfval_alert_t *ap   = NULL;    /* alert pointer */
  spfval_alert_t *atmp = NULL;    /* temporary pointer */


  if (ctx == NULL)
  {
    xvprint("NULL arguments passed");

    return;
  }

  ap = ctx->alert_head;

  while (ap != NULL)
  {
    atmp = ap;
    ap   = ap->next;

    xfree(atmp->hint);
    xfree(atmp);
  }

  xfree(ctx->record);
  xfree(ctx);
  
  return;
}


/* SPFVAL_validate
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Performs SPFv1 validation on the passed context.
*
*  Date:   November 18, 2004
*
*  Desc:
*          Altered return type to by SPFVAL_BOOL and appropriately
*  adjusted all return statements.
*
*/
SPFVAL_BOOL SPFVAL_validate(spfval_ctx_t *ctx)
{
  char *p = NULL;                       /* working pointer */

  char token[MAX_RECORD_LEN] = {0};     /* token string */


  if (ctx == NULL)
  {
    xvprint("NULL arguments passed");

    return(SPFVAL_FALSE);
  }

  if (!_SPFVAL_is_spf_record(ctx))
  {
    _SPFVAL_error(ctx, ERR_NOT_SPF_RECORD);

    return(SPFVAL_FALSE);
  }

  p = ctx->record;

  /* find the start of the first token */
  while (*p == ' ')
  {
    ctx->offset++;
    p++;
  }

  xvprintf("Starting validation procedure on record: %s", p);

  while (*p && _SPFVAL_get_token(ctx, &p, token))
  {
    if (token[0] == 0)
    {
      xvprint("Obtained an empty token");

      break;
    }

    ctx->num_directives++;
    _SPFVAL_token(ctx, token);

    if (*p)
    {
      ctx->offset++;
    }
  }

  if (ctx->num_directives < 2)
  {
    xvprint("Record has no effect");

    _SPFVAL_error(ctx, ERR_RECORD_HAS_NO_EFFECT);

    return(SPFVAL_FALSE);
  }

  if (ctx->num_mechanisms == 0)
  {
    if (!ctx->found_redirect)
    {
      xvprint("Not enough mechanisms to be a valid SPFv1 record");

      _SPFVAL_error(ctx, ERR_NOT_ENOUGH_MECHANISMS);

      return(SPFVAL_FALSE);
    }
  }

  xvprint("Record parse completed");

  if (ctx->num_errors)
  {
    return(SPFVAL_FALSE);
  }
  
  return(SPFVAL_TRUE);
}


/* SPFVAL_count_alerts
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Tally's up alerts
*
*  Return:
*           SPFVAL_FALSE upon discovery of an invalid context, or
*  a positive integer reflecting the number of alerts encountered
*  which could include 0.
*
*/
int32_t SPFVAL_count_alerts(spfval_ctx_t *ctx)
{
  if (ctx == NULL)
  {
    return(SPFVAL_FALSE);
  }

  return(ctx->num_alerts);
}


/* SPFVAL_count_errors
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   August 16, 2004 
*
*  Desc:
*          Tally's up errors.
*       
*  Return:
*           SPFVAL_FALSE upon discovery of an invalid context, or
*  a positive integer reflecting the number of errors encountered
*  which could include 0.
*
*/
int32_t SPFVAL_count_errors(spfval_ctx_t *ctx)
{
  if (ctx == NULL)
  {
    return(SPFVAL_FALSE);
  }

  return(ctx->num_errors);
}


/* SPFVAL_count_warnings
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Tally's up warnings.
*
*  Return:
*           SPFVAL_FALSE upon discovery of an invalid context, or
*  a positive integer reflecting the number of warning encountered
*  which could include 0.
*
*/
int32_t SPFVAL_count_warnings(spfval_ctx_t *ctx)
{
  if (ctx == NULL)
  {
    return(SPFVAL_FALSE);
  }

  return(ctx->num_warnings);
}


/* SPFVAL_get_alert
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  Date:   August 16, 2004 
*
*  Desc:
*          Obtains an alert from the supplied context.
*
*  Return: 
*          A pointer to an alert upon success, and NULL upon
*  failure to obtain one.
*
*/
spfval_alert_t *SPFVAL_get_alert(spfval_ctx_t *ctx)
{
  if (ctx == NULL)
  {
    return(NULL);
  }

  if (ctx->alert_head == NULL)
  {
    return(NULL);
  }

  if (ctx->alert_ptr == NULL)
  {
    ctx->alert_ptr = ctx->alert_head;
  }
  else if (ctx->alert_ptr == ctx->alert_tail)
  {
    ctx->alert_ptr = NULL;
  }
  else
  {
    ctx->alert_ptr = ctx->alert_ptr->next;
  }

  return(ctx->alert_ptr);
}


/* end spfval.c */
