/* DDT - Distributed Domain Traversal
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  File:   spfval.h
*  Desc:   spfval.c Header File
*
*  License:
*
*  The CodeShare Software License, Version 1.0
*
*  Copyright (c) 2004 James Couzens & Travis Anderson  All rights
*  reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
*  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED.  IN NO EVENT SHALL CODESHARE NOR ANY OF ITS ASSOCIATED
*  PROGRAMMERS OR OTHER INDIVIDUALS AFFILIATED WITH THE OWNERSHIP OR
*  OPERATION OF CODESHARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
*  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
*  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
*  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*
*/


#ifndef _SPFVAL_H
#define _SPFVAL_H 1

#include "util.h"     /* DDT common utility functions */


#ifndef _WIN32
__BEGIN_DECLS
#endif /* _WIN32 */



/*
*  define declarations
*
*/


#define MAX_RECORD_LEN 1024       /* maximum record length */
#define VERSION_STRING "spf1"     /* SPF version string */
#define SPFVAL_VERSION "0.0.1"    /* spfval version string */



/* 
*  enumeration declarations
*
*/


/* SPFVAL_BOOL
*
*  Desc:
*         A simple TRUE/FALSE boolean declared using a specific name
*  so as to not collide with anything else possibly existing within the
*  namespace.  (Its quite common to find TRUE/FALSE defined within the 
*  namespace and sadly I've seen them declared in either order 0/1 and
*  1/0 and thus can not reliably be used.)
*    
*/
typedef enum
{
  SPFVAL_FALSE = 0,
  SPFVAL_TRUE
} SPFVAL_BOOL;


/* SPFVAL_ALERT_TYPE
*
*  Desc: 
*         Alert types
*
*/
typedef enum
{
  ALERT_ERROR = 0,      /**/
  ALERT_WARNING,        /**/
  ALERT_END             /* number of aler types */
} SPFVAL_ALERT_TYPE;


/* SPFVAL_ERROR_ID
*
*  Desc:  
*         ID error codes
*
*/
typedef enum
{
  ERR_NOT_SPF_RECORD = 0,                /**/
  ERR_NOT_ENOUGH_MECHANISMS,             /**/
  ERR_MODIFIER_WITH_PREFIX,              /**/
  ERR_MECHANISM_UNRECOGNIZED,            /**/
  ERR_DIRECTIVE_CANNOT_CONTAIN_MACRO,    /**/
  ERR_MACRO_NOT_TERMINATED,              /**/
  ERR_MACRO_NOT_VALID_IN_MECHANISM,      /**/
  ERR_INVALID_MACRO_COMMAND,             /**/
  ERR_INVALID_MACRO_TRANSFORMER,         /**/
  ERR_INVALID_MACRO_DELIMITER,           /**/
  ERR_TRAILING_CHARS_ON_MACRO,           /**/
  ERR_MODIFIER_HAS_NO_ARGUMENT,          /**/
  ERR_CIDR_NOT_APPLICABLE,               /**/
  ERR_EXISTS_MUST_HAVE_ARGUMENT,         /**/
  ERR_ARGUMENT_NOT_APPLICABLE,           /**/
  ERR_RECORD_HAS_NO_EFFECT,              /**/
  ERR_EMPTY_MACRO,                       /**/
  ERR_END                                /* number of error strings */
} SPFVAL_ERROR_ID;


/* SPFVAL_WARNING_ID
*
*  Desc:
*         enumerated list of warning ID's
*
*/
typedef enum
{
  WARN_MODIFIER_DOESNT_CONFORM = 0,    /**/
  WARN_UNESCAPED_PERCENT,              /* unescaped % char */
  WARN_END                             /* number of warnings */
} SPFVAL_WARNING_ID;


/* SPFVAL_MECHANISM
*
*  Desc: 
*         enumerated list of SPF mechanisms
*
*/
typedef enum
{
  MECH_ALL = 0,        /* 'all' */
  MECH_INCLUDE,        /* 'include' */
  MECH_A,              /* 'a' */
  MECH_MX,             /* 'mx' */
  MECH_PTR,            /* 'ptr' */
  MECH_IP4,            /* 'ip4' */
  MECH_IP6,            /* 'ip6' */
  MECH_EXISTS,         /* 'exists' */
  MECH_END             /* number of mechanisms */
} SPFVAL_MECHANISM;


/* SPFVAL_MODIFIER
*
*  Desc: 
*         enumerated list of SPF modifiers
*
*/
typedef enum
{
  MOD_VERSION = 0,    /* 'v=spfx' where x is version as int32_t */
  MOD_REDIRECT,       /* 'redirect' */
  MOD_EXPLAIN,        /* 'explain' */
  MOD_END             /*  number of modifiers */
} SPFVAL_MODIFIER;



/*
*  Structure declarations
*
*/


/* spfval_alert_t
*
*  Desc:
*         A structure designed to be placed int32_to a linked list
*  and obtained through a set of helper functions when their
*  contents are desired.
*
*/
typedef struct spfval_alert_s
{
  char *type;                     /* type of alert */
  char *str;                      /* alert text */
  char *hint;                     /* remainder of original rec at offset */

  int32_t code;                   /* alert code */
  int32_t offset;                 /* offset value */

  struct spfval_alert_s *next;    /* next element in list */
} spfval_alert_t;


/* spfval_ctx_t
*
*  Desc:
*         ctx is short for context and this is effectively the primary
*  information structure within spfval and contains all the relevant 
*  data that is resultant of an SPF record validation attempt.
*
*/
typedef struct spfval_ctx_s
{
  char *record;                  /* string to validate */
  
  int32_t record_len;            /* length of record */
  int32_t offset;                /* position in record */

  int32_t num_directives;        /* directives count */
  int32_t num_mechanisms;        /* mechanisms count */
  int32_t num_modifiers;         /* modifiers count */
  int32_t num_alerts;            /* alerts count */
  int32_t num_errors;            /* errors count */
  int32_t num_warnings;          /* warnings count */
 
  SPFVAL_BOOL found_include;     /* include present */
  SPFVAL_BOOL found_redirect;    /* redirect present */

  spfval_alert_t *alert_ptr;     /* pointer to alert */
  spfval_alert_t *alert_head;    /* head of alert list */
  spfval_alert_t *alert_tail;    /* yail of alert list */
} spfval_ctx_t;



/*
*  global declarations
*
*/




/*
*  function declarations 
*
*/

/* private functions */

/*
static SPFVAL_BOOL _SPFVAL_is_prefix(const char);
static SPFVAL_BOOL _SPFVAL_is_macro_letter(const char);
static int32_t _SPFVAL_is_macro_transformer(const char);
static SPFVAL_BOOL _SPFVAL_is_macro_delimiter(const char);
static int32_t _SPFVAL_get_mechanism_id(const char *);
static int32_t _SPFVAL_get_modifier_id(const char *);
static int32_t _SPFVAL_alert(spfval_ctx_t *, int32_t, int32_t);
static SPFVAL_BOOL _SPFVAL_error(spfval_ctx_t *, int32_t);
static SPFVAL_BOOL _SPFVAL_warning(spfval_ctx_t *, int32_t);
static int32_t _SPFVAL_macro(spfval_ctx_t *, const char *, const char *);
static int32_t _SPFVAL_get_macro(spfval_ctx_t *, char **, char *);
static int32_t _SPFVAL_modifier(spfval_ctx_t *, const char *, const char *, int32_t);
static int32_t _SPFVAL_mechanism(spfval_ctx_t *, const char *, const char *, int32_t);
static int32_t _SPFVAL_token(spfval_ctx_t *, const char *);
static int32_t _SPFVAL_get_token(spfval_ctx_t *, char **, char *);
static int32_t _SPFVAL_is_spf_record(spfval_ctx_t *);
static SPFVAL_BOOL _SPFVAL_cidr_cmp(const policy_addr_t *,
                                    const struct in_addr *);

*/


/*
* public functions
*
*/

spfval_ctx_t   *SPFVAL_new(const char *);
void            SPFVAL_free(spfval_ctx_t *);
SPFVAL_BOOL     SPFVAL_validate(spfval_ctx_t *);
int32_t         SPFVAL_count_alerts(spfval_ctx_t *);
int32_t         SPFVAL_count_errors(spfval_ctx_t *);
int32_t         SPFVAL_count_warnings(spfval_ctx_t *);
spfval_alert_t *SPFVAL_get_alert(spfval_ctx_t *);


#ifndef _WIN32
__END_DECLS
#endif /* _WIN32 */

#endif /* _SPFVAL_H */

/* end spfval.h */
