/* Sender Policy Framework Validator
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  FILE: util.c
*  DESC: Source file for utility functions.
*
*  License:
*
*  The CodeShare Software License, Version 1.0
*
*  Copyright (c) 2004 James Couzens & Travis Anderson.  All rights
*  reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
*  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED.  IN NO EVENT SHALL THE CODESHARE ORGANIZATION OR
*  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
*  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
*  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
*  SUCH DAMAGE.
*
*/


#include <errno.h>     /* errno */
#include <stdarg.h>    /* va_* */
#include <stdio.h>     /* printf */
#include <stdlib.h>    /* malloc / free */
#include <string.h>    /* memcpy, sprintf, snprintf */
#include <time.h>      /* time_t */

#include "util.h"      /* Our header file */


/*
* local defines 
*
*/

#define SPFVAL_MAX_TS  20


/* _UTIL_malloc
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   02/17/04
*
*  Desc:
*         Wrapper for malloc.  Upon success, behaves as malloc does.
*  Wrapper functionality is to print an error message and exit upon failure.
*
*/
void *_UTIL_malloc(const int32_t n, const char *file, int32_t line,
                  const char *func)
{
  void *x = NULL;    /* pointer to allocated memory */
  
  
  if ((x = malloc(n)) == NULL)
  {
    xprintf("Unable to allocate %i bytes at %s:%i in %s",
      n, file, line, func);

    exit(1);
  }

  return(x);
}


/* _UTIL_realloc
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   02/17/04
*
*  Desc:
*         Wrapper for realloc.  If 'p' is NULL, allocates memory via a call to
*  _UTIL_malloc, otherwise if 'x' is assigned successfully, the behaviour is
*  identical to realloc.  Wrapper functionality is to print an error message
*  and exit upon failure.
*
*/
void *_UTIL_realloc(void *p, const int32_t n, const char *file, const int32_t line,
                    const char *func)
{
  void *x = NULL;    /* pointer to reallocaed memory */


  if (p == NULL)
  {
    return(_UTIL_malloc(n, file, line, func));
  }

  x = realloc(p, n);
  if (x == NULL)
  {
    xprintf("Unable to reallocate %i bytes at %s:%i in %s; address origin 0x%x",
      n, file, line, func, p);

    exit(1);
  }

  return(x);
}


/* _UTIL_free
*
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   02/17/04
*
*  Desc:
*         Wrapper for free.  Upon success, behaves as free does.
*  Wrapper functionality is to print an error message and exit upon
*  failure.
*
*/
void _UTIL_free(void *p, const char *file, const int32_t line, const char *func)
{
  if (p == NULL)
  {
    xvprintf("Unable to free() on NULL pointer at %s:%i "
             "in %s; address 0x%x.", file, line, func, (u_int32_t)p);
    return;
  }

  xvprintf("Free address 0x%x by %s on line %i (%s)",
           (u_int32_t)p, func, line, file);

  free(p);

  return;
}


/* _UTIL_printf
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*  Date:   12/25/03
*
*  Desc:
*          Prints debug output.
*
*  Date:   11/18/04 - James Couzens <jcouzens@codeshare.ca>
*
*  Desc:
*          Removed magic numbers and replaced with macros, added comments
*  and switched 'DEBUG' for '_SPFVAL_DEBUG', also applied default values
*  to chars and pointers.
*
*/
void _UTIL_printf(const int32_t level, const char *file, const int32_t line,
                  const char *function, const char *format, ...)
{
  va_list arg_ptr;                   /* arguments list passed to us*/
  
  time_t ltime;                      /* local time */
  
  struct tm *today;                  /* time */
  
  char ts[SPFVAL_MAX_TS]   = {0};    /* timestamp container */
  char str[SPFVAL_MAX_STR] = {0};    /* string buffer */

  char *fptr = NULL;                 /**/


#ifndef _SPFVAL_DEBUG
  if (level == LOG_VERBOSE)
  {
    return;
  }
#endif /* _SPFVAL_DEBUG */

  va_start (arg_ptr, format);
  vsnprintf (str, SPFVAL_MAX_STR, format, arg_ptr);
  va_end (arg_ptr);

  time(&ltime);
  today = localtime(&ltime);

  strftime(ts, SPFVAL_MAX_TS, "%m/%d/%Y %T", today);

#ifdef _WIN32
  fptr = strrchr(file, '\\');
#else
  fptr = strrchr(file, '/');
#endif /* _WIN32 */

  if (fptr == NULL)
  {
    fptr = (char *)file;
  }
  else
  {
    fptr++;
  }

#ifdef _SPFVAL_DEBUG
  printf("[%s] %s:%i %s: %s\n", ts, fptr, line, function, str);
#else
  printf("[%s] %s\n", ts, str);
#endif /* _SPFVAL_DEBUG */

  return;
}


/* _UTIL_strndup
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*
*  Date:   12/25/03
*
*  Desc:
*          n bytes are allocated and then filled with \0 chars.  Char s
*  is copied over to the allocated memory writing n -1 bytes leaving the
*  new string NULL terminated.  This new string is returned upon success
*  and NULL upon failure.
*
*/
char *_UTIL_strndup(const char *s, const size_t n)
{                                                
  size_t i = 0;            /* length of s */

  char *ret_ptr = NULL;    /* return buffer */
        

  if (!s || (n <= 0))
  {
    xvprintf("[%i] Passed string is NULL.  Abort!.\n", i);
        
    return(NULL);
  }

  xvprintf("called with string: (%s) of len: %i\n", s, n);

  if ((i = (strlen(s) + 1)) > n)
  {
    ret_ptr = xmalloc(n);
    xvprintf("Allocated %u bytes of memory.\n", n);
    snprintf(ret_ptr, n, s);
  }
  else
  {
    ret_ptr = xmalloc(i);
    xvprintf("Allocated %u bytes of memory.\n", i);
    snprintf(ret_ptr, i, s);
  }

  xvprintf("leaving function; returning string: (%s)\n", ret_ptr);

  return(ret_ptr);
}


/* end of util.c */
