/* Sender Policy Framework Validator
*
*  Author: James Couzens <jcouzens@codeshare.ca>
*  Author: Travis Anderson <tanderson@codeshare.ca>
*
*  FILE: util.h
*  DESC: Header file for utility functions.
*
*  License:
*
*  The CodeShare Software License, Version 1.0
*
*  Copyright (c) 2004 James Couzens & Travis Anderson.  All rights
*  reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
*  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
*  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED.  IN NO EVENT SHALL THE CODESHARE ORGANIZATION OR
*  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
*  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
*  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
*  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
*  SUCH DAMAGE.
*
*/


#ifndef _UTIL_H
#define _UTIL_H 1

#include <stdlib.h>       /* malloc / free */
#include <netinet/in.h>   /* in_addr struct fBSD */
#include <arpa/inet.h>    /* in_addr struct */


#ifndef _WIN32
__BEGIN_DECLS
#endif /* _WIN32 */



/*
*  define declarations 
*
*/

/* maximum string length */
#define SPFVAL_MAX_STR 4096

/* logging levels */
#define LOG_NORMAL  1
#define LOG_VERBOSE 2

/* malloc wrapper macros */
#define xmalloc(n)     _UTIL_malloc(n, __FILE__, __LINE__, __FUNCTION__)
#define xrealloc(m, n) _UTIL_realloc(m, n, __FILE__, __LINE__, __FUNCTION__)
#define xfree(m)       _UTIL_free(m, __FILE__, __LINE__, __FUNCTION__)

/* string function wrapper macros */
#define xstrndup(m, n) _UTIL_strndup(m, n)

/* regular output macros */
#define xprint(s) \
  _UTIL_printf(LOG_NORMAL, __FILE__, __LINE__, __FUNCTION__, (s), "")

#define xprintf(format,...) \
  _UTIL_printf(LOG_NORMAL, __FILE__, __LINE__, __FUNCTION__, format, \
  __VA_ARGS__)

/* verbose output macros */
#define xvprint(s) \
  _UTIL_printf(LOG_VERBOSE, __FILE__, __LINE__, __FUNCTION__, (s), "")

#define xvprintf(format,...) \
  _UTIL_printf(LOG_VERBOSE, __FILE__, __LINE__, __FUNCTION__, format, \
  __VA_ARGS__)


#ifdef _SPFVAL_DEBUG

  /* xreturn wrapper */
  #define xreturn(m) \
  { \
    xvprintf("Returning '%s'", #m); \
    \
    return (m); \
  }

  /* xenter wrapper */
  #define xenter() (xvprint("Entering"))

  /* xassert wrapper */
  #define xassert(expr)                               \
  if ((expr))                                         \
  {                                                   \
    xvprintf("Assertion '%s' succeeded", #expr);      \
  }                                                   \
  else                                                \
  {                                                   \
    xprintf("Assertion '%s' failed; exiting", #expr); \
                                                      \
    exit(1);                                          \
  }

#else

  /* xreturn wrapper */
  #define xreturn(m) \
  { \
    return (m); \
  }

  /* xenter wrapper */
  #define xenter()

  /* xassert wrapper */
  #define xassert(expr)


#endif /* _SPFVAL_DEBUG */

#ifdef _SPFVAL_DEBUG
#else
#endif



/*
* structure declarations
*
*/


/* policy_addr_t
*
*  Desc:  
*         Storage container used to store parsed out ip addresses in their
*  binary format (in_addr struct) and an unsigned integer containing the 
*  netmask
*
*/
typedef struct policy_addr_s
{
  struct in_addr addr;    /* in_addr struct (unsigned long) */
  uint8_t        cidr;    /* address cidr length */
} policy_addr_t;




/*
*  function declarations 
*
*/


/* alloc wrapper functions */
void *_UTIL_malloc(const int32_t, const char *, const int32_t, const char *);
void *_UTIL_realloc(void *, const int32_t, const char *, const int32_t,
                    const char *);
void  _UTIL_free(void *, const char *, const int32_t, const char *);
char *_UTIL_strndup(const char *, const size_t);


/* printf wrapper function */
void  _UTIL_printf(const int32_t, const char *, const int32_t, const char *,
                   const char *format, ...);

                  
#ifndef _WIN32
__END_DECLS
#endif /* _WIN32 */

#endif /* _UTIL_H */

/* end util.h */
